/*
*****************************************************************************
**

**  File        : LinkerScript.ld
**
**  Abstract    : Linker script for STM32L073VZ Device with
**                192KByte FLASH, 20KByte RAM
**
**                Set heap size, stack size and stack location according
**                to application requirements.
**
**                Set memory bank area and size if external memory is used.
**
**  Target      : STMicroelectronics STM32
**
**
**  Distribution: The file is distributed as is, without any warranty
**                of any kind.
**
**  (c)Copyright Ac6.
**  You may use this file as-is or modify it according to the needs of your
**  project. Distribution of this file (unmodified or modified) is not
**  permitted. Ac6 permit registered System Workbench for MCU users the
**  rights to distribute the assembled, compiled & linked contents of this
**  file as part of an application binary file, provided that it is built
**  using the System Workbench for MCU toolchain.
**
*****************************************************************************
*/
/* Entry Point */
ENTRY(Reset_Handler)

_estack = 0x20005000;    /* end of RAM */
/* Generate a link error if heap and stack don't fit into RAM */
_Min_Heap_Size = 0x200;;      /* required amount of heap  */
_Min_Stack_Size = 0x400;; /* required amount of stack */

/* Specify the memory areas */
MEMORY
{
FLASH (rx)      : ORIGIN = 0x8000000, LENGTH = 192K
RAM (xrw)      : ORIGIN = 0x20000000, LENGTH = 20K


/*  NVDATA_PROTECTED_CODE: used to locate code in protected code segment    */
/*                                                                          */
/****************************************************************************/
/* Note that the protected code segment doesn't exactly start at the 
   starting address used to configure de Firewall.
   This allows to locate the function used to open the Firewall at the
   correct address to obey the call gate sequence */
 
  NVDATA_PROTECTED_CODE (rx)   : ORIGIN = 0x08004004, LENGTH = 512
/***************************************************************************/

/*                                                                         */
/*             NVDATA_PROTECTED_DATA: used to locate data in               */
/*                     non-volatile data segment                           */
/*                                                                         */
/***************************************************************************/    
  NVDATA_PROTECTED_DATA (xrw)  : ORIGIN = 0x08005000 , LENGTH = 256 
/***************************************************************************/

/***************************************************************************/
/*                                                                         */
/*    VDATA_PROTECTED_DATA: used to locate data in volatile data segment   */
/*                                                                         */
/***************************************************************************/    
  VDATA_PROTECTED_DATA (xrw)  : ORIGIN = 0x20000200, LENGTH = 64    
  
}




/* Define output sections */
SECTIONS
{
  /* The startup code goes first into FLASH */
  .isr_vector :
  {
    . = ALIGN(4);
    KEEP(*(.isr_vector)) /* Startup code */
    . = ALIGN(4);
  } >FLASH

  /* The program code and other data goes into FLASH */
  .text :
  {
    . = ALIGN(4);
    *(.text)           /* .text sections (code) */
    *(.text*)          /* .text* sections (code) */
    *(.glue_7)         /* glue arm to thumb code */
    *(.glue_7t)        /* glue thumb to arm code */
    *(.eh_frame)

    KEEP (*(.init))
    KEEP (*(.fini))

    . = ALIGN(4);
    _etext = .;        /* define a global symbols at end of code */
  } >FLASH

  /* Constant data goes into FLASH */
  .rodata :
  {
    . = ALIGN(4);
    *(.rodata)         /* .rodata sections (constants, strings, etc.) */
    *(.rodata*)        /* .rodata* sections (constants, strings, etc.) */
    . = ALIGN(4);
  } >FLASH

  .ARM.extab   : { *(.ARM.extab* .gnu.linkonce.armextab.*) } >FLASH
  .ARM : {
    __exidx_start = .;
    *(.ARM.exidx*)
    __exidx_end = .;
  } >FLASH

  .preinit_array     :
  {
    PROVIDE_HIDDEN (__preinit_array_start = .);
    KEEP (*(.preinit_array*))
    PROVIDE_HIDDEN (__preinit_array_end = .);
  } >FLASH
  .init_array :
  {
    PROVIDE_HIDDEN (__init_array_start = .);
    KEEP (*(SORT(.init_array.*)))
    KEEP (*(.init_array*))
    PROVIDE_HIDDEN (__init_array_end = .);
  } >FLASH
  .fini_array :
  {
    PROVIDE_HIDDEN (__fini_array_start = .);
    KEEP (*(SORT(.fini_array.*)))
    KEEP (*(.fini_array*))
    PROVIDE_HIDDEN (__fini_array_end = .);
  } >FLASH

  /* used by the startup to initialize data */
  _sidata = LOADADDR(.data);

  /* Initialized data sections goes into RAM, load LMA copy after code */
  .data : 
  {
    . = ALIGN(4);
    _sdata = .;        /* create a global symbol at data start */
    *(.data)           /* .data sections */
    *(.data*)          /* .data* sections */

    . = ALIGN(4);
    _edata = .;        /* define a global symbol at data end */
  } >RAM AT> FLASH

  
  /* Uninitialized data section */
  . = ALIGN(4);
  .bss :
  {
    /* This is used by the startup in order to initialize the .bss secion */
    _sbss = .;         /* define a global symbol at bss start */
    __bss_start__ = _sbss;
    *(.bss)
    *(.bss*)
    *(COMMON)

    . = ALIGN(4);
    _ebss = .;         /* define a global symbol at bss end */
    __bss_end__ = _ebss;
  } >RAM

  /* User_heap_stack section, used to check that there is enough RAM left */
  ._user_heap_stack :
  {
    . = ALIGN(4);
    PROVIDE ( end = . );
    PROVIDE ( _end = . );
    . = . + _Min_Heap_Size;
    . = . + _Min_Stack_Size;
    . = ALIGN(4);
  } >RAM

  /*************************************************************************/   
  /*                                                                       */
  /* NVDATA_PROTECTED_CODE section, code is located here explicitly        */
  /* refer to protected_code.c:                                            */
  /*                                                                       */    
  /* void __attribute__((section(".nvdata_protected_code"))) protected_function(void) */
  /*                                                                       */
  /*************************************************************************/      
  .memory_nvdata_protected_code :
  {
    *(.nvdata_protected_code)      /* .nvdata_protected_code sections (code)  */
    *(.nvdata_protected_code*)     /* .nvdata_protected_code* sections (code) */
  } >NVDATA_PROTECTED_CODE


  /*************************************************************************/   
  /*                                                                       */
  /* NVDATA_PROTECTED_DATA section, data is located here explicitly        */
  /* refer to protected_nonvolatile_data.c:                                */
  /* const uint32_t __attribute__((section(".nvdata_protected_data")))  protected_non_volatile_array[N] = {90, 91, 92, 93, 94, 95, 96, 97, 98, 99}; */
  /*                                                                       */
  /*************************************************************************/      
  .memory_nvdata_protected_data :
  {
    *(.nvdata_protected_data)     /* .nvdata_protected_data sections (code)   */
    *(.nvdata_protected_data*)    /* .nvdata_protected_data* sections (code)  */
  } >NVDATA_PROTECTED_DATA
  
  /*************************************************************************/   
  /*                                                                       */
  /* VDATA_PROTECTED_DATA section, data is located here explicitly         */
  /* refer to protected_volatile_data.c:                                   */
  /* uint32_t __attribute__((section(".vdata_protected_data")))  protected_volatile_array[N] = {50, 51, 52, 53, 54, 55, 56, 57, 58, 59}; */
  /*                                                                       */
  /*************************************************************************/      
  .memory_vdata_protected_data :
  {
    *(.vdata_protected_data)        /* .vdata_protected_data sections (code)  */
    *(.vdata_protected_data*)       /* .vdata_protected_data* sections (code) */
  } >VDATA_PROTECTED_DATA    
  

  /* Remove information from the standard libraries */
  /DISCARD/ :
  {
    libc.a ( * )
    libm.a ( * )
    libgcc.a ( * )
  }

  .ARM.attributes 0 : { *(.ARM.attributes) }
}

